% Michael Vodhanel

% Purpose: Whitens observables to be processed by the ICA algorithm

% Input:
%	X: The unwhitened observables 

% Output:
%	Xnew: The whitened observables
%	m: The means (transformed) of the whitened observables
%	E: The matrix of eigenvectors of X
%	D: The diagonal matrix of eigenvalues of X

function [Xnew, m, E, D] = Preprocess(X)

Xnew = zeros(size(X));

% Subtract the mean
m = mean(X, 2);
for i = 1:size(X, 1)
    Xnew(i,:) = X(i,:) - m(i);
end

% Whiten Data
[E, D, E_trans] = svd(Xnew*Xnew');
Xnew = E * inv(sqrt(D)) * E' * Xnew;
Id = Xnew * Xnew';

% Transform m
m = E * inv(sqrt(D)) * E' * m;

%UnWhitener = E * inv(sqrt(D)) * E';